-- DROP DATABASE test_utf8;
CREATE DATABASE test_utf8 DEFAULT CHARSET = utf8mb4;
USE test_utf8;


GRANT ALL PRIVILEGES ON test_utf8.* TO test@localhost IDENTIFIED BY 'Test_1234' WITH GRANT OPTION; #-- FOR V5.x
GRANT ALL PRIVILEGES ON test_utf8.* TO test@'%' IDENTIFIED BY 'Test_1234' WITH GRANT OPTION; #-- FOR V5.x
GRANT ALL PRIVILEGES ON test_utf8.* TO test@'127.0.0.1' IDENTIFIED BY 'Test_1234' WITH GRANT OPTION; #-- FOR V5.x

# CREATE USER 'test@localhost' IDENTIFIED WITH MYSQL_NATIVE_PASSWORD BY 'Test_1234'; #-- FOR V8.x
# GRANT ALL PRIVILEGES ON test_utf8.* TO 'test@localhost'; #-- FOR V8.x
# CREATE USER 'test@%' IDENTIFIED WITH MYSQL_NATIVE_PASSWORD BY 'Test_1234'; #-- FOR V8.x
# GRANT ALL PRIVILEGES ON test_utf8.* TO 'test@%'; #-- FOR V8.x
# CREATE USER 'test@127.0.0.1' IDENTIFIED WITH MYSQL_NATIVE_PASSWORD BY 'Test_1234'; #-- FOR V8.x
# GRANT ALL PRIVILEGES ON test_utf8.* TO 'test@127.0.0.1'; #-- FOR V8.x

FLUSH PRIVILEGES;

SET PASSWORD FOR 'test'@'localhost'=PASSWORD('Test_1234'); #-- FOR V5.x
SET PASSWORD FOR 'test'@'%'=PASSWORD('Test_1234'); #-- FOR V5.x
SET PASSWORD FOR 'test'@'127.0.0.1'=PASSWORD('Test_1234'); #-- FOR V5.x

# ALTER USER 'test@localhost' IDENTIFIED WITH MYSQL_NATIVE_PASSWORD BY 'Test_1234'; #-- FOR V8.x
# ALTER USER 'test@%' IDENTIFIED WITH MYSQL_NATIVE_PASSWORD BY 'Test_1234'; #-- FOR V8.x
# ALTER USER 'test@127.0.0.1' IDENTIFIED WITH MYSQL_NATIVE_PASSWORD BY 'Test_1234'; #-- FOR V8.x

FLUSH PRIVILEGES;

SET  FOREIGN_KEY_CHECKS = 0;


-- 組織マスタ
DROP TABLE IF EXISTS `organizations`;
CREATE TABLE `organizations` (
  `id` INT(8) UNSIGNED NOT NULL AUTO_INCREMENT COMMENT '組織ID',
  `parent_id` INT(8) UNSIGNED DEFAULT NULL COMMENT '親組織ID',
  `level` ENUM('administrator', 'head_office', 'master_agent', 'agent') NOT NULL DEFAULT 'agent' COMMENT 'レベル',
  `name` VARCHAR(64) NOT NULL COMMENT '組織名',
  `balance` DECIMAL(16, 8) NOT NULL DEFAULT '0' COMMENT '残高',
  `status` ENUM('active', 'inactive', 'pending') NOT NULL DEFAULT 'pending' COMMENT '状態',
  `created_id` INT(8) UNSIGNED DEFAULT NULL COMMENT '作成者',
  `created_at` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP COMMENT '作成日時',
  `updated_id` INT(8) UNSIGNED DEFAULT NULL COMMENT '更新者',
  `updated_at` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP COMMENT '更新日時',
  `deleted_id` INT(8) UNSIGNED DEFAULT NULL COMMENT '削除者',
  `deleted_at` DATETIME DEFAULT NULL COMMENT '削除日時',
  `logical` TINYINT(1) GENERATED ALWAYS AS (CASE WHEN `deleted_at` IS NULL THEN 1 ELSE NULL END) STORED COMMENT '論理値',
  PRIMARY KEY (`id`),
  KEY `organizations_idx_1` (`name`),
  CONSTRAINT `organizations_uk_1` UNIQUE KEY (`name`, `logical`),
  CONSTRAINT `organizations_fk_1` FOREIGN KEY (`parent_id`) REFERENCES `organizations` (`id`) ON DELETE SET NULL ON UPDATE SET NULL
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 ROW_FORMAT=COMPACT
COMMENT='組織マスタ';


INSERT INTO
`organizations`
(
`id`,
`parent_id`,
`level`,
`name`,
`balance`,
`status`,
`created_id`,
`created_at`,
`updated_id`,
`updated_at`,
`deleted_id`,
`deleted_at`
)
VALUES
(
1,
NULL,
'administrator',
'administrator',
'0',
'active',
'1',
CURRENT_TIMESTAMP(),
'1',
CURRENT_TIMESTAMP(),
NULL,
NULL
),
(
2,
1,
'head_office',
'HEAD',
'0',
'active',
'1',
CURRENT_TIMESTAMP(),
'1',
CURRENT_TIMESTAMP(),
NULL,
NULL
),
(
3,
2,
'master_agent',
'MASTER',
'0',
'active',
'1',
CURRENT_TIMESTAMP(),
'1',
CURRENT_TIMESTAMP(),
NULL,
NULL
),
(
4,
3,
'agent',
'AGENT',
'0',
'active',
'1',
CURRENT_TIMESTAMP(),
'1',
CURRENT_TIMESTAMP(),
NULL,
NULL
);


-- ユーザーマスタ
DROP TABLE IF EXISTS `users`;
CREATE TABLE `users` (
  `id` INT(8) UNSIGNED NOT NULL AUTO_INCREMENT COMMENT 'ユーザーID',
  `organization_id` INT(8) UNSIGNED NOT NULL COMMENT ' 組織ID',
  `is_master` TINYINT(1) NOT NULL DEFAULT '0' COMMENT 'マスターアカウント識別フラグ',
  `username` VARCHAR(64) NOT NULL COMMENT 'ユーザー名',
  `password` VARCHAR(192) NOT NULL COMMENT 'パスワード',
  `first_name` VARCHAR(32) NOT NULL DEFAULT '' COMMENT '名前(名)',
  `last_name` VARCHAR(32) NOT NULL DEFAULT '' COMMENT '苗字(姓)',
  `full_name` VARCHAR(64) GENERATED ALWAYS AS (CONCAT(COALESCE(`first_name`, ''), ' ', COALESCE(`last_name`, ''))) VIRTUAL NOT NULL COMMENT 'フルネーム',
  `email` VARCHAR(128) NOT NULL DEFAULT '' COMMENT 'Eメールアドレス',
  `tel` VARCHAR(16) NOT NULL DEFAULT '' COMMENT '電話番号',
  `pin` VARCHAR(4) NOT NULL DEFAULT '0000' COMMENT '暗証番号',
  `ip_address` VARCHAR(64) NOT NULL DEFAULT '' COMMENT 'IPアドレス',
  `user_agent` VARCHAR(256) NOT NULL DEFAULT '' COMMENT 'ユーザーエージェント',
  `finger_print` VARCHAR(64) NOT NULL DEFAULT '' COMMENT 'フィンガープリント',
  `last_login` DATETIME DEFAULT NULL COMMENT '最終ログイン',
  `failed_login_attempt` TINYINT(1) NOT NULL DEFAULT 0 COMMENT 'ログイン失敗回数',
  `config` JSON NOT NULL COMMENT '設定',
  `status` ENUM('active', 'inactive', 'pending') NOT NULL DEFAULT 'pending' COMMENT '状態',
  `created_id` INT(8) UNSIGNED DEFAULT NULL COMMENT '作成者',
  `created_at` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP COMMENT '作成日時',
  `updated_id` INT(8) UNSIGNED DEFAULT NULL COMMENT '更新者',
  `updated_at` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP COMMENT '更新日時',
  `deleted_id` INT(8) UNSIGNED DEFAULT NULL COMMENT '削除者',
  `deleted_at` DATETIME DEFAULT NULL COMMENT '削除日時',
  `logical` TINYINT(1) GENERATED ALWAYS AS (CASE WHEN `deleted_at` IS NULL THEN 1 ELSE NULL END) STORED COMMENT '論理値',
  PRIMARY KEY (`id`),
  KEY `users_idx_1` (`username`),
  CONSTRAINT `users_uk_1` UNIQUE KEY (`username`, `logical`),
  CONSTRAINT `users_uk_2` UNIQUE KEY (`email`, `logical`),
  CONSTRAINT `users_uk_3` UNIQUE KEY (`tel`, `logical`),
  CONSTRAINT `users_fk_1` FOREIGN KEY (`organization_id`) REFERENCES `organizations` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 ROW_FORMAT=COMPACT
COMMENT='ユーザーマスタ';


INSERT INTO
`users`
(
`id`,
`organization_id`,
`is_master`,
`username`,
`password`,
`first_name`,
`last_name`,
`email`,
`tel`,
`pin`,
`ip_address`,
`user_agent`,
`finger_print`,
`last_login`,
`failed_login_attempt`,
`config`,
`status`,
`created_id`,
`created_at`,
`updated_id`,
`updated_at`,
`deleted_id`,
`deleted_at`
)
VALUES
(
1,
1,
1,
'administrator',
'$2a$08$1qE.Vo1utv9KVxk4vp9VfunkzajAaXxFmjZLoW3LeXXws4UVIlUei', -- 28524174AbXe
'administrator',
'administrator',
'administrator@root.com',
'00000000000',
'0000',
'127.0.0.1',
'',
'',
NULL,
0,
JSON_OBJECT(),
'active',
'0',
CURRENT_TIMESTAMP(),
'0',
CURRENT_TIMESTAMP(),
NULL,
NULL
),
(
2,
2,
1,
'HEAD',
'$2a$08$1qE.Vo1utv9KVxk4vp9VfunkzajAaXxFmjZLoW3LeXXws4UVIlUei', -- 28524174AbXe
'HEAD',
'HEAD',
'head@test.com',
'11111111111',
'0000',
'127.0.0.1',
'',
'',
NULL,
0,
JSON_OBJECT(),
'active',
'1',
CURRENT_TIMESTAMP(),
'1',
CURRENT_TIMESTAMP(),
NULL,
NULL
),
(
3,
3,
1,
'MASTER',
'$2a$08$1qE.Vo1utv9KVxk4vp9VfunkzajAaXxFmjZLoW3LeXXws4UVIlUei', -- 28524174AbXe
'MASTER',
'MASTER',
'master@test.com',
'22222222222',
'0000',
'127.0.0.1',
'',
'',
NULL,
0,
JSON_OBJECT(),
'active',
'2',
CURRENT_TIMESTAMP(),
'2',
CURRENT_TIMESTAMP(),
NULL,
NULL
),
(
4,
4,
1,
'AGENT',
'$2a$08$1qE.Vo1utv9KVxk4vp9VfunkzajAaXxFmjZLoW3LeXXws4UVIlUei', -- 28524174AbXe
'AGENT',
'AGENT',
'agent@test.com',
'33333333333',
'0000',
'127.0.0.1',
'',
'',
NULL,
0,
JSON_OBJECT(),
'active',
'3',
CURRENT_TIMESTAMP(),
'3',
CURRENT_TIMESTAMP(),
NULL,
NULL
);

DROP TABLE IF EXISTS `tests`;
CREATE TABLE `tests` (
  `id` INT(8) UNSIGNED NOT NULL AUTO_INCREMENT COMMENT 'ID',
  `name` VARCHAR(64) NOT NULL COMMENT '名前',
  `status` ENUM('active', 'inactive', 'pending') NOT NULL DEFAULT 'pending' COMMENT '状態',
  `created_id` INT(8) UNSIGNED DEFAULT NULL COMMENT '作成者',
  `created_at` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP COMMENT '作成日時',
  `updated_id` INT(8) UNSIGNED DEFAULT NULL COMMENT '更新者',
  `updated_at` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP COMMENT '更新日時',
  `deleted_id` INT(8) UNSIGNED DEFAULT NULL COMMENT '削除者',
  `deleted_at` DATETIME DEFAULT NULL COMMENT '削除日時',
  `logical` TINYINT(1) GENERATED ALWAYS AS (CASE WHEN `deleted_at` IS NULL THEN 1 ELSE NULL END) STORED COMMENT '論理値',
  PRIMARY KEY (`id`),
  KEY `tests_idx_1` (`name`),
  CONSTRAINT `tests_uk_1` UNIQUE KEY (`name`, `logical`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 ROW_FORMAT=COMPACT
COMMENT='テスト';


SET  FOREIGN_KEY_CHECKS = 1;
