'use strict'
// @ts-check

const _ = require('lodash')
const {
  Validate,
  Validated,
  ValidationResult,
} = require('@fizz.js/node-validate/libs')
const ResponseFormatter = require('@fizz.js/node-response-formatter/libs')
const Models = require('../models')
const CONSTANT = require('@fizz.js/node-constant/libs')

/**
 *
 *
 * @class {{CLAZZ}}Controller
 */
class {{CLAZZ}}Controller {
  /**
   *
   * @param {string} [method='']
   * @returns {express.Middleware[]}
   * @memberof {{CLAZZ}}Controller
   */
  validationMiddleware(method = '') {
    return Models.{{MODEL}}.getValidators(method)
  }

  /**
   *
   *
   * @param {any} req
   * @param {any} res
   * @param {any} next
   * @returns
   *
   * @memberof {{CLAZZ}}Controller
   */
  async getAll(req, res, next) {
    try {
      const input = Validated(req)
      const errors = ValidationResult(req)
      if (!errors.isEmpty()) {
        return errors.throw()
      }

      const sort = _.toLower(`${input.sort || 'id'}`)
      const order = _.toUpper(`${input.order || 'DESC'}`)
      const filter = _.toLower(`${input.filter || ''}`)
      const where = {
        status: [CONSTANT.STATUS.ACTIVE, CONSTANT.STATUS.INACTIVE, CONSTANT.STATUS.PENDING]
      }
      if (!_.isEmpty(filter)) {
        where[Models.op.or] = Models.whereFieldsLike(['id', 'status'], filter)
      }

      const condition = {
        where: _.pickBy(where, _.identity),
        order: [
          [sort, order]
        ]
      }

      const {
        count = 0
      } = await Models.{{MODEL}}.findAndCountAll(condition)
      const total = _.toNumber(count || 0)
      const limit = _.toNumber(input.limit || 10)
      const page = _.toNumber(input.page || 1)
      const pages = _.ceil(total / limit)
      const offset = limit * (page - 1) || 0

      condition.limit = limit
      condition.offset = offset

      /** @type {Array<Model{{CLAZZ}}>} */
      const result = await Models.{{MODEL}}.findAll(condition)
      const rows = result.map(item => item.dataValues)

      return ResponseFormatter.success(req, res, next, {
        rows,
        paginate: {
          total,
          page,
          pages,
          limit,
          sort,
          offset,
          filter,
          order
        }
      })
    } catch (error) {
      return ResponseFormatter.raiseError(error, req, res, next)
    }
  }

  /**
   *
   *
   * @param {any} req
   * @param {any} res
   * @param {any} next
   * @returns
   *
   * @memberof {{CLAZZ}}Controller
   */
  async getOne(req, res, next) {
    try {
      const input = Validated(req)
      const errors = ValidationResult(req)
      if (!errors.isEmpty()) {
        return errors.throw()
      }

      const { id } = input
      const status = CONSTANT.STATUS.ACTIVE
      const where = {
        id,
        status
      }

      /** @type {Model{{CLAZZ}}} */
      const { dataValues } = await Models.{{MODEL}}.findOne({
        where: _.pickBy(where, _.identity)
      })

      return ResponseFormatter.success(req, res, next, dataValues)
    } catch (error) {
      return ResponseFormatter.raiseError(error, req, res, next)
    }
  }

  /**
   *
   *
   * @param {any} req
   * @param {any} res
   * @param {any} next
   * @returns
   *
   * @memberof {{CLAZZ}}Controller
   */
  async create(req, res, next) {
    let tx
    try {
      const input = Validated(req)
      const errors = ValidationResult(req)
      if (!errors.isEmpty()) {
        return errors.throw()
      }

      tx = await Models.startTransaction()
      const options = {
        transaction: tx,
        lock: tx.LOCK.UPDATE
      }

      const {
        status = CONSTANT.STATUS.ACTIVE,
      } = input
      const userId = _.get(req, 'user.id')
      const createdId = userId
      const updatedId = userId
      const payload = {
        status,
        createdId,
        updatedId,
      }

      /** @type {Model{{CLAZZ}}} */
      const model = await Models.{{MODEL}}.create(payload, options)
      const { id = undefined } = model

      await tx.commit()
      tx = null

      const {
        dataValues
      } = await Models.{{MODEL}}.findByPk(id, {
        useMaster: true
      })

      return ResponseFormatter.success(req, res, next, dataValues)
    } catch (error) {
      if (tx) {
        await tx.rollback()
      }

      return ResponseFormatter.raiseError(error, req, res, next)
    } finally {
      await Models.endTransaction()
    }
  }

  /**
   *
   *
   * @param {any} req
   * @param {any} res
   * @param {any} next
   * @returns
   *
   * @memberof {{CLAZZ}}Controller
   */
  async update(req, res, next) {
    let tx
    try {
      const input = Validated(req)
      const errors = ValidationResult(req)
      if (!errors.isEmpty()) {
        return errors.throw()
      }

      tx = await Models.startTransaction()
      const options = {
        transaction: tx,
        lock: tx.LOCK.UPDATE
      }

      const {
        id
      } = input
      const userId = _.get(req, 'user.id')
      const updatedId = userId
      const status = CONSTANT.STATUS.INACTIVE
      const model = await Models.{{MODEL}}.findByPk(id, options)
      const {
        dataValues
      } = await model.update({
        status,
        updatedId,
      }, options)

      await tx.commit()
      tx = null

      return ResponseFormatter.success(req, res, next, dataValues)
    } catch (error) {
      if (tx) {
        await tx.rollback()
      }

      return ResponseFormatter.raiseError(error, req, res, next)
    } finally {
      await Models.endTransaction()
    }
  }

  /**
   *
   *
   * @param {any} req
   * @param {any} res
   * @param {any} next
   * @returns
   *
   * @memberof {{CLAZZ}}Controller
   */
  async destroy(req, res, next) {
    try {
      const input = Validated(req)
      const errors = ValidationResult(req)
      if (!errors.isEmpty()) {
        return errors.throw()
      }

      const {
        id
      } = input
      const model = await Models.{{MODEL}}.findByPk(id)
      await model.destroy()

      return ResponseFormatter.success(req, res, next)
    } catch (error) {
      return ResponseFormatter.raiseError(error, req, res, next)
    }
  }
}

module.exports = {{CLAZZ}}Controller
